@@ -128,6 +128,8 @@ struct aws_s3_client_vtable {
 
     void (
         *setup_vip_connection_retry_token)(struct aws_s3_client *client, struct aws_s3_vip_connection *vip_connection);
+
+    void (*finish_destroy)(struct aws_s3_client *client);
 };
 
 /* Represents the state of the S3 client. */
@@ -209,15 +211,18 @@ struct aws_s3_client {
     struct {
         struct aws_mutex lock;
 
-        /* Endpoint to use for the bucket. */
-        struct aws_string *endpoint;
-
         /* How many vips are being actively used. */
         uint32_t active_vip_count;
 
         /* How many vips are allocated. (This number includes vips that are in the process of cleaning up) */
         uint32_t allocated_vip_count;
 
+        /* How many requests failed to be prepared. */
+        uint32_t num_failed_prepare_requests;
+
+        /* Endpoint to use for the bucket. */
+        struct aws_string *endpoint;
+
         /* Linked list of active VIP's. */
         struct aws_linked_list vips;
 
@@ -377,7 +382,6 @@ void aws_s3_client_unlock_synced_data(struct aws_s3_client *client);
 
 AWS_S3_API
 extern const uint32_t g_max_num_connections_per_vip;
-
 AWS_EXTERN_C_END
 
 #endif /* AWS_S3_CLIENT_IMPL_H */
