@@ -14,6 +14,9 @@
 
 enum aws_s3_errors {
     AWS_ERROR_S3_MISSING_CONTENT_RANGE_HEADER = AWS_ERROR_ENUM_BEGIN_RANGE(AWS_C_S3_PACKAGE_ID),
+    AWS_ERROR_S3_INVALID_CONTENT_RANGE_HEADER,
+    AWS_ERROR_S3_MISSING_CONTENT_LENGTH_HEADER,
+    AWS_ERROR_S3_INVALID_CONTENT_LENGTH_HEADER,
     AWS_ERROR_S3_MISSING_ETAG,
     AWS_ERROR_S3_INTERNAL_ERROR,
     AWS_ERROR_S3_SLOW_DOWN,
@@ -24,6 +27,8 @@ enum aws_s3_errors {
     AWS_ERROR_S3_UNSUPPORTED_PROXY_SCHEME,
     AWS_ERROR_S3_CANCELED,
     AWS_ERROR_S3_NO_ENDPOINT_CONNECTIONS,
+    AWS_ERROR_S3_INVALID_RANGE_HEADER,
+    AWS_ERROR_S3_MULTIRANGE_HEADER_UNSUPPORTED,
 
     AWS_ERROR_S3_END_RANGE = AWS_ERROR_ENUM_END_RANGE(AWS_C_S3_PACKAGE_ID)
 };
