@@ -147,6 +147,7 @@ struct aws_s3_tester_meta_request_options {
     /* Get Object Meta Request specific options.*/
     struct {
         struct aws_byte_cursor object_path;
+        struct aws_byte_cursor object_range;
     } get_options;
 
     /* Put Object Meta request specific options. */
@@ -348,4 +349,9 @@ extern const struct aws_byte_cursor g_test_bucket_name;
 extern const struct aws_byte_cursor g_test_public_bucket_name;
 extern const struct aws_byte_cursor g_s3_path_get_object_test_1MB;
 
+extern const struct aws_byte_cursor g_pre_existing_object_1MB;
+extern const struct aws_byte_cursor g_pre_existing_object_kms_10MB;
+extern const struct aws_byte_cursor g_pre_existing_object_aes256_10MB;
+extern const struct aws_byte_cursor g_pre_existing_empty_object;
+
 #endif /* AWS_S3_TESTER_H */
