@@ -37,8 +37,6 @@
 #include "tls/s2n_connection.h"
 #include "utils/s2n_safety.h"
 
-#define S2N_MAX_ECC_CURVE_NAME_LENGTH 10
-
 void usage()
 {
     fprintf(stderr, "usage: s2nc [options] host [port]\n");
@@ -85,17 +83,14 @@ void usage()
     fprintf(stderr, "    Turn on corked io\n");
     fprintf(stderr, "  -B,--non-blocking\n");
     fprintf(stderr, "    Set the non-blocking flag on the connection's socket.\n");
-    fprintf(stderr, "  -K ,--keyshares\n");
-    fprintf(stderr, "    Colon separated list of curve names.\n"
-                    "    The client will generate keyshares only for the curve names present in the ecc_preferences list configured in the security_policy.\n"
-                    "    The curves currently supported by s2n are: `x25519`, `secp256r1` and `secp384r1`. Note that `none` represents a list of empty keyshares.\n"
-                    "    By default, the client will generate keyshares for all curves present in the ecc_preferences list.\n");
     fprintf(stderr, "  -L --key-log <path>\n");
     fprintf(stderr, "    Enable NSS key logging into the provided path\n");
     fprintf(stderr, "  -P --psk <psk-identity,psk-secret,psk-hmac-alg> \n"
                     "    A comma-separated list of psk parameters in this order: psk_identity, psk_secret and psk_hmac_alg.\n"
                     "    Note that the maximum number of permitted psks is 10, the psk-secret is hex-encoded, and whitespace is not allowed before or after the commas.\n"
                     "    Ex: --psk psk_id,psk_secret,SHA256 --psk shared_id,shared_secret,SHA384.\n");
+    fprintf(stderr, "  -E ,--early-data <file path>\n");
+    fprintf(stderr, "    Sends data in file path as early data to the server. Early data will only be sent if s2nc receives a session ticket and resumes a session.");
     fprintf(stderr, "\n");
     exit(1);
 }
@@ -120,6 +115,27 @@ static uint8_t unsafe_verify_host(const char *host_name, size_t host_name_len, v
     return (uint8_t) (strcasecmp(host_name, verify_data->trusted_host) == 0);
 }
 
+size_t session_state_length = 0;
+uint8_t *session_state = NULL;
+static int test_session_ticket_cb(struct s2n_connection *conn, void *ctx, struct s2n_session_ticket *ticket)
+{
+    GUARD_EXIT_NULL(conn);
+    GUARD_EXIT_NULL(ticket);
+
+    GUARD_EXIT(s2n_session_ticket_get_data_len(ticket, &session_state_length), "Error getting ticket length ");
+    session_state = realloc(session_state, session_state_length);
+    if(session_state == NULL) {
+        print_s2n_error("Error getting new session state");
+        exit(1);
+    }
+    GUARD_EXIT(s2n_session_ticket_get_data(ticket, session_state_length, session_state), "Error getting ticket data");
+
+    bool *session_ticket_recv = (bool *)ctx;
+    *session_ticket_recv = 1;
+
+    return S2N_SUCCESS;
+}
+
 static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs, s2n_status_request_type type,
     struct verify_data *unsafe_verify_data, const char *host, const char *alpn_protocols, uint16_t mfl_value) {
 
@@ -231,8 +247,7 @@ int main(int argc, char *const *argv)
 {
     struct addrinfo hints, *ai_list, *ai;
     int r, sockfd = 0;
-    ssize_t session_state_length = 0;
-    uint8_t *session_state = NULL;
+    bool session_ticket_recv = 0;
     /* Optional args */
     const char *alpn_protocols = NULL;
     const char *server_name = NULL;
@@ -257,14 +272,11 @@ int main(int argc, char *const *argv)
     int echo_input = 0;
     int use_corked_io = 0;
     uint8_t non_blocking = 0;
-    int keyshares_count = 0;
-    char keyshares[S2N_ECC_EVP_SUPPORTED_CURVES_COUNT][S2N_MAX_ECC_CURVE_NAME_LENGTH];
-    char *input = NULL;
-    char *token = NULL;
     const char *key_log_path = NULL;
     FILE *key_log_file = NULL;
     char *psk_optarg_list[S2N_MAX_PSK_LIST_LENGTH];
     size_t psk_list_len = 0;
+    char *early_data = NULL;
 
     static struct option long_options[] = {
         {"alpn", required_argument, 0, 'a'},
@@ -285,16 +297,16 @@ int main(int argc, char *const *argv)
         {"timeout", required_argument, 0, 't'},
         {"corked-io", no_argument, 0, 'C'},
         {"tls13", no_argument, 0, '3'},
-        {"keyshares", required_argument, 0, 'K'},
         {"non-blocking", no_argument, 0, 'B'},
         {"key-log", required_argument, 0, 'L'},
         {"psk", required_argument, 0, 'P'},
+        {"early-data", required_argument, 0, 'E'},
         { 0 },
     };
 
     while (1) {
         int option_index = 0;
-        int c = getopt_long(argc, argv, "a:c:ehn:m:sf:d:l:k:D:t:irTCK:BL:P:", long_options, &option_index);
+        int c = getopt_long(argc, argv, "a:c:ehn:m:sf:d:l:k:D:t:irTCBL:P:E:", long_options, &option_index);
         if (c == -1) {
             break;
         }
@@ -314,17 +326,6 @@ int main(int argc, char *const *argv)
         case 'h':
             usage();
             break;
-        case 'K':
-            input = optarg;
-            token = strtok(input, ":");
-            while( token != NULL ) {
-                strcpy(keyshares[keyshares_count], token);
-                if (++keyshares_count == S2N_ECC_EVP_SUPPORTED_CURVES_COUNT) {
-                    break;
-                }
-                token = strtok(NULL, ":");
-            }
-            break;
         case 'n':
             server_name = optarg;
             break;
@@ -383,6 +384,10 @@ int main(int argc, char *const *argv)
             }
             psk_optarg_list[psk_list_len++] = optarg;
             break;
+        case 'E':
+            early_data = load_file_to_cstring(optarg);
+            GUARD_EXIT_NULL(early_data);
+            break;
         case '?':
         default:
             usage();
@@ -478,6 +483,8 @@ int main(int argc, char *const *argv)
 
         if (session_ticket) {
             GUARD_EXIT(s2n_config_set_session_tickets_onoff(config, 1), "Error enabling session tickets");
+            GUARD_EXIT(s2n_config_set_session_ticket_cb(config, test_session_ticket_cb, &session_ticket_recv), "Error setting session ticket callback");
+            session_ticket_recv = 0;
         }
 
         if (key_log_path) {
@@ -512,18 +519,24 @@ int main(int argc, char *const *argv)
             GUARD_EXIT(s2n_connection_use_corked_io(conn), "Error setting corked io");
         }
 
-        for (size_t i = 0; i < keyshares_count; i++) {
-            if (keyshares[i]) {
-                GUARD_EXIT(s2n_connection_set_keyshare_by_name_for_testing(conn, keyshares[i]), "Error setting keyshares to generate");
-            }
-        }
-
         /* Update session state in connection if exists */
         if (session_state_length > 0) {
             GUARD_EXIT(s2n_connection_set_session(conn, session_state, session_state_length), "Error setting session state in connection");
         }
 
-        GUARD_EXIT(s2n_setup_external_psk_list(conn, psk_optarg_list, psk_list_len), "Error setting external psk list"); 
+        GUARD_EXIT(s2n_setup_external_psk_list(conn, psk_optarg_list, psk_list_len), "Error setting external psk list");
+
+        if (early_data) {
+            if (!session_ticket) {
+                print_s2n_error("Early data can only be used with session tickets.");
+                exit(1);
+            }
+            /* Send early data if we have a received a session ticket from the server */
+            if (session_state_length) {
+                uint32_t early_data_length = strlen(early_data);
+                GUARD_EXIT(early_data_send(conn, (uint8_t *)early_data, early_data_length), "Error sending early data");
+            }
+        }
 
         /* See echo.c */
         if (negotiate(conn, sockfd) != 0) {
@@ -533,18 +546,27 @@ int main(int argc, char *const *argv)
 
         printf("Connected to %s:%s\n", host, port);
 
-        /* Save session state from connection if reconnect is enabled */
+        /* Save session state from connection if reconnect is enabled. */
         if (reconnect > 0) {
-            if (!session_ticket && s2n_connection_get_session_id_length(conn) <= 0) {
-                printf("Endpoint sent empty session id so cannot resume session\n");
-                exit(1);
-            }
-            free(session_state);
-            session_state_length = s2n_connection_get_session_length(conn);
-            session_state = calloc(session_state_length, sizeof(uint8_t));
-            if (s2n_connection_get_session(conn, session_state, session_state_length) != session_state_length) {
-                print_s2n_error("Error getting serialized session state");
-                exit(1);
+            if (conn->actual_protocol_version >= S2N_TLS13) {
+                if (!session_ticket) {
+                    print_s2n_error("s2nc can only reconnect in TLS1.3 with session tickets.");
+                    exit(1);
+                }
+                GUARD_EXIT(echo(conn, sockfd, &session_ticket_recv), "Error calling echo");
+            } else {
+                if (!session_ticket && s2n_connection_get_session_id_length(conn) <= 0) {
+                    print_s2n_error("Endpoint sent empty session id so cannot resume session");
+                    exit(1);
+                }
+                free(session_state);
+                session_state_length = s2n_connection_get_session_length(conn);
+                session_state = calloc(session_state_length, sizeof(uint8_t));
+                GUARD_EXIT_NULL(session_state);
+                if (s2n_connection_get_session(conn, session_state, session_state_length) != session_state_length) {
+                    print_s2n_error("Error getting serialized session state");
+                    exit(1);
+                }
             }
         }
 
@@ -552,10 +574,13 @@ int main(int argc, char *const *argv)
             s2n_connection_set_dynamic_record_threshold(conn, dyn_rec_threshold, dyn_rec_timeout);
         }
 
+        GUARD_EXIT(s2n_connection_free_handshake(conn), "Error freeing handshake memory after negotiation");
+
         if (echo_input == 1) {
+            bool stop_echo = false;
             fflush(stdout);
             fflush(stderr);
-            echo(conn, sockfd);
+            echo(conn, sockfd, &stop_echo);
         }
 
         /* The following call can block on receiving a close_notify if we initiate the shutdown or if the */
@@ -584,6 +609,7 @@ int main(int argc, char *const *argv)
 
     GUARD_EXIT(s2n_cleanup(), "Error running s2n_cleanup()");
 
+    free(early_data);
     free(session_state);
     freeaddrinfo(ai_list);
     return 0;
