@@ -637,7 +637,7 @@ static const char* tls13_handshake_type_names[] = {
 #define CONNECTION_WRITER( conn ) (conn->mode == S2N_CLIENT ? 'C' : 'S')
 #define CONNECTION_IS_WRITER( conn ) (ACTIVE_STATE(conn).writer == CONNECTION_WRITER(conn))
 
-/* Used in our test cases */
+/* Only used in our test cases. */
 message_type_t s2n_conn_get_current_message_type(struct s2n_connection *conn)
 {
     return ACTIVE_MESSAGE(conn);
@@ -666,12 +666,12 @@ static int s2n_advance_message(struct s2n_connection *conn)
      * we don't mess with it
      */
     if (!conn->corked_io || s2n_socket_was_corked(conn)) {
-        return 0;
+        return S2N_SUCCESS;
     }
 
     /* Are we changing I/O directions */
     if (ACTIVE_STATE(conn).writer == previous_writer || ACTIVE_STATE(conn).writer == 'A') {
-        return 0;
+        return S2N_SUCCESS;
     }
 
     /* We're the new writer */
@@ -681,7 +681,7 @@ static int s2n_advance_message(struct s2n_connection *conn)
             POSIX_GUARD(s2n_socket_write_cork(conn));
         }
 
-        return 0;
+        return S2N_SUCCESS;
     }
 
     /* We're the new reader, or we reached the "B" writer stage indicating that
@@ -690,7 +690,7 @@ static int s2n_advance_message(struct s2n_connection *conn)
         POSIX_GUARD(s2n_socket_write_uncork(conn));
     }
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 int s2n_generate_new_client_session_id(struct s2n_connection *conn)
@@ -703,7 +703,7 @@ int s2n_generate_new_client_session_id(struct s2n_connection *conn)
         conn->session_id_len = S2N_TLS_SESSION_ID_MAX_LEN;
     }
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 /* Lets the server flag whether a HelloRetryRequest is needed while processing extensions */
@@ -733,7 +733,9 @@ int s2n_set_hello_retry_required(struct s2n_connection *conn)
 
 bool s2n_is_hello_retry_message(struct s2n_connection *conn)
 {
-    return (ACTIVE_MESSAGE(conn) == HELLO_RETRY_MSG);
+    return (conn != NULL &&
+            s2n_result_is_ok(s2n_handshake_validate(&(conn->handshake))) &&
+            ACTIVE_MESSAGE(conn) == HELLO_RETRY_MSG);
 }
 
 bool s2n_is_hello_retry_handshake(struct s2n_connection *conn)
@@ -807,8 +809,8 @@ int s2n_conn_set_handshake_type(struct s2n_connection *conn)
 
     if (conn->config->use_tickets) {
         if (conn->session_ticket_status == S2N_DECRYPT_TICKET) {
-            if (!s2n_decrypt_session_ticket(conn)) {
-                return 0;
+            if (!s2n_decrypt_session_ticket(conn, &conn->client_ticket_to_decrypt)) {
+                return S2N_SUCCESS;
             }
 
             if (s2n_config_is_encrypt_decrypt_key_available(conn->config) == 1) {
@@ -829,14 +831,14 @@ int s2n_conn_set_handshake_type(struct s2n_connection *conn)
      * Client sent in the ClientHello. */
     if (conn->actual_protocol_version <= S2N_TLS12 && conn->mode == S2N_SERVER && s2n_allowed_to_cache_connection(conn)) {
         int r = s2n_resume_from_cache(conn);
-        if (r == S2N_SUCCESS || (r < 0 && S2N_ERROR_IS_BLOCKING(s2n_errno))) {
+        if (r == S2N_SUCCESS || (r < S2N_SUCCESS && S2N_ERROR_IS_BLOCKING(s2n_errno))) {
             return r;
         }
     }
 
 skip_cache_lookup:
     if (conn->mode == S2N_CLIENT && conn->client_session_resumed == 1) {
-        return 0;
+        return S2N_SUCCESS;
     }
 
     /* If we're doing full handshake, generate a new session id. */
@@ -866,21 +868,22 @@ int s2n_conn_set_handshake_no_client_cert(struct s2n_connection *conn)
 
     POSIX_GUARD_RESULT(s2n_handshake_type_set_flag(conn, NO_CLIENT_CERT));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 const char *s2n_connection_get_last_message_name(struct s2n_connection *conn)
 {
     PTR_ENSURE_REF(conn);
-
+    PTR_GUARD_RESULT(s2n_handshake_validate(&(conn->handshake)));
     return message_names[ACTIVE_MESSAGE(conn)];
 }
 
 const char *s2n_connection_get_handshake_type_name(struct s2n_connection *conn)
 {
     PTR_ENSURE_REF(conn);
+    PTR_PRECONDITION(s2n_handshake_validate(&(conn->handshake)));
 
-    uint16_t handshake_type = conn->handshake.handshake_type;
+    uint32_t handshake_type = conn->handshake.handshake_type;
 
     if (handshake_type == INITIAL) {
         return "INITIAL";
@@ -973,7 +976,7 @@ static int s2n_handshake_write_io(struct s2n_connection *conn)
     /* Advance the state machine */
     POSIX_GUARD(s2n_advance_message(conn));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 /*
@@ -1036,7 +1039,7 @@ static int s2n_handshake_conn_update_hashes(struct s2n_connection *conn)
     /* MD5 and SHA sum the handshake data too */
     POSIX_GUARD(s2n_conn_update_handshake_hashes(conn, &handshake_record));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 static int s2n_handshake_handle_sslv2(struct s2n_connection *conn)
@@ -1062,32 +1065,15 @@ static int s2n_handshake_handle_sslv2(struct s2n_connection *conn)
     /* We're done with the record, wipe it */
     POSIX_GUARD(s2n_stuffer_wipe(&conn->header_in));
     POSIX_GUARD(s2n_stuffer_wipe(&conn->in));
-    if (r < 0) {
-        /* Don't invoke blinding on some of the common errors */
-        switch (s2n_errno) {
-            case S2N_ERR_CANCELLED:
-            case S2N_ERR_CIPHER_NOT_SUPPORTED:
-            case S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED:
-                conn->closed = 1;
-                break;
-            case S2N_ERR_IO_BLOCKED:
-            case S2N_ERR_ASYNC_BLOCKED:
-                /* A blocking condition is retryable, so we should return without killing the connection. */
-                S2N_ERROR_PRESERVE_ERRNO();
-                break;
-            default:
-                POSIX_GUARD(s2n_connection_kill(conn));
-        }
 
-        return r;
-    }
+    WITH_ERROR_BLINDING(conn, POSIX_GUARD(r));
 
     conn->in_status = ENCRYPTED;
 
     /* Advance the state machine */
     POSIX_GUARD(s2n_advance_message(conn));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 static int s2n_try_delete_session_cache(struct s2n_connection *conn)
@@ -1098,7 +1084,7 @@ static int s2n_try_delete_session_cache(struct s2n_connection *conn)
         conn->config->cache_delete(conn, conn->config->cache_delete_data, conn->session_id, conn->session_id_len);
     }
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 static S2N_RESULT s2n_wipe_record(struct s2n_connection *conn)
@@ -1110,6 +1096,17 @@ static S2N_RESULT s2n_wipe_record(struct s2n_connection *conn)
     return S2N_RESULT_OK;
 }
 
+static S2N_RESULT s2n_finish_read(struct s2n_connection *conn)
+{
+    RESULT_ENSURE_REF(conn);
+
+    RESULT_GUARD_POSIX(s2n_handshake_conn_update_hashes(conn));
+    RESULT_GUARD_POSIX(s2n_stuffer_wipe(&conn->handshake.io));
+    RESULT_GUARD_POSIX(s2n_tls13_handle_secrets(conn));
+    RESULT_GUARD_POSIX(s2n_advance_message(conn));
+    return S2N_RESULT_OK;
+}
+
 /* Reading is a little more complicated than writing as the TLS RFCs allow content
  * types to be interleaved at the record layer. We may get an alert message
  * during the handshake phase, or messages of types that we don't support (e.g.
@@ -1146,9 +1143,7 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
          *# "early_data" extension, it MUST terminate the connection with a
          *# "bad_record_mac" alert as per Section 5.2.
          */
-        if ((r < 0) && (s2n_errno == S2N_ERR_DECRYPT) /* Decryption Error */
-                && (conn->mode == S2N_SERVER) /* On the server */
-                && (conn->early_data_state == S2N_EARLY_DATA_REJECTED) /* When early data was rejected */) {
+        if ((r < S2N_SUCCESS) && (s2n_errno == S2N_ERR_EARLY_DATA_TRIAL_DECRYPT)) {
             POSIX_GUARD(s2n_stuffer_reread(&conn->in));
             POSIX_GUARD_RESULT(s2n_early_data_record_bytes(conn, s2n_stuffer_data_available(&conn->in)));
             POSIX_GUARD_RESULT(s2n_wipe_record(conn));
@@ -1194,7 +1189,7 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
             POSIX_GUARD(s2n_advance_message(conn));
         }
 
-        return 0;
+        return S2N_SUCCESS;
     } else if (record_type != TLS_HANDSHAKE) {
         if (record_type == TLS_ALERT) {
             POSIX_GUARD(s2n_process_alert_fragment(conn));
@@ -1245,44 +1240,26 @@ static int s2n_handshake_read_io(struct s2n_connection *conn)
             POSIX_GUARD_RESULT(s2n_handshake_type_unset_tls12_flag(conn, OCSP_STATUS));
         }
 
+        /*
+         *= https://tools.ietf.org/rfc/rfc5246#section-7.4
+         *# The one message that is not bound by these ordering rules
+         *# is the HelloRequest message, which can be sent at any time, but which
+         *# SHOULD be ignored by the client if it arrives in the middle of a handshake.
+         */
+        if (message_type == TLS_HELLO_REQUEST) {
+            POSIX_GUARD(s2n_client_hello_request_recv(conn));
+            return S2N_SUCCESS;
+        }
+
         POSIX_ENSURE(record_type == EXPECTED_RECORD_TYPE(conn), S2N_ERR_BAD_MESSAGE);
         POSIX_ENSURE(message_type == EXPECTED_MESSAGE_TYPE(conn), S2N_ERR_BAD_MESSAGE);
         POSIX_ENSURE(!CONNECTION_IS_WRITER(conn), S2N_ERR_BAD_MESSAGE);
 
         /* Call the relevant handler */
-        r = ACTIVE_STATE(conn).handler[conn->mode] (conn);
-
-        /* Don't update handshake hashes until after the handler has executed since some handlers need to read the
-         * hash values before they are updated. */
-        POSIX_GUARD(s2n_handshake_conn_update_hashes(conn));
-
-        POSIX_GUARD(s2n_stuffer_wipe(&conn->handshake.io));
-
-        if (r < 0) {
-            /* Don't invoke blinding on some of the common errors */
-            switch (s2n_errno) {
-                case S2N_ERR_CANCELLED:
-                case S2N_ERR_CIPHER_NOT_SUPPORTED:
-                case S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED:
-                    conn->closed = 1;
-                    break;
-                case S2N_ERR_IO_BLOCKED:
-                case S2N_ERR_ASYNC_BLOCKED:
-                    /* A blocking condition is retryable, so we should return without killing the connection. */
-                    S2N_ERROR_PRESERVE_ERRNO();
-                    break;
-                default:
-                    POSIX_GUARD(s2n_connection_kill(conn));
-            }
-
-            return r;
-        }
-
-        /* Update the secrets, if necessary */
-        POSIX_GUARD(s2n_tls13_handle_secrets(conn));
+        WITH_ERROR_BLINDING(conn, POSIX_GUARD(ACTIVE_STATE(conn).handler[conn->mode] (conn)));
 
         /* Advance the state machine */
-        POSIX_GUARD(s2n_advance_message(conn));
+        POSIX_GUARD_RESULT(s2n_finish_read(conn));
     }
 
     /* We're done with the record, wipe it */
@@ -1299,7 +1276,7 @@ static int s2n_handle_retry_state(struct s2n_connection *conn)
     s2n_errno = S2N_ERR_OK;
     const int r = ACTIVE_STATE(conn).handler[conn->mode] (conn);
 
-    if (r < 0 && S2N_ERROR_IS_BLOCKING(s2n_errno)) {
+    if (r < S2N_SUCCESS && S2N_ERROR_IS_BLOCKING(s2n_errno)) {
         /* If the handler is still waiting for data, return control to the caller. */
         S2N_ERROR_PRESERVE_ERRNO();
     }
@@ -1314,30 +1291,26 @@ static int s2n_handle_retry_state(struct s2n_connection *conn)
         conn->in_status = ENCRYPTED;
     }
 
-    if (r < 0) {
-        /* There is some other problem and we should kill the connection. */
-        if (conn->session_id_len) {
-            s2n_try_delete_session_cache(conn);
-        }
-
-        POSIX_GUARD(s2n_connection_kill(conn));
-        S2N_ERROR_PRESERVE_ERRNO();
-    }
-
     if (CONNECTION_IS_WRITER(conn)) {
+        POSIX_GUARD(r);
+
         /* If we're the writer and handler just finished, update the record header if
          * needed and let the s2n_handshake_write_io write the data to the socket */
         if (EXPECTED_RECORD_TYPE(conn) == TLS_HANDSHAKE) {
             POSIX_GUARD(s2n_handshake_finish_header(&conn->handshake.io));
         }
     } else {
+        if (r < S2N_SUCCESS && conn->session_id_len) {
+            s2n_try_delete_session_cache(conn);
+        }
+        WITH_ERROR_BLINDING(conn, POSIX_GUARD(r));
+
         /* The read handler processed the record successfully, we are done with this
          * record. Advance the state machine. */
-        POSIX_GUARD(s2n_tls13_handle_secrets(conn));
-        POSIX_GUARD(s2n_advance_message(conn));
+        POSIX_GUARD_RESULT(s2n_finish_read(conn));
     }
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 int s2n_negotiate(struct s2n_connection *conn, s2n_blocked_status *blocked)
@@ -1362,7 +1335,7 @@ int s2n_negotiate(struct s2n_connection *conn, s2n_blocked_status *blocked)
             *blocked = S2N_BLOCKED_ON_WRITE;
             const int write_result = s2n_handshake_write_io(conn);
 
-            if (write_result < 0) {
+            if (write_result < S2N_SUCCESS) {
                 if (!S2N_ERROR_IS_BLOCKING(s2n_errno)) {
                     /* Non-retryable write error. The peer might have sent an alert. Try and read it. */
                     const int write_errno = errno;
@@ -1394,7 +1367,7 @@ int s2n_negotiate(struct s2n_connection *conn, s2n_blocked_status *blocked)
             *blocked = S2N_BLOCKED_ON_READ;
             const int read_result = s2n_handshake_read_io(conn);
 
-            if (read_result < 0) {
+            if (read_result < S2N_SUCCESS) {
                 /* One blocking condition is waiting on the session resumption cache. */
                 /* So we don't want to delete anything if we are blocked. */
                 if (!S2N_ERROR_IS_BLOCKING(s2n_errno) && conn->session_id_len) {
@@ -1412,16 +1385,16 @@ int s2n_negotiate(struct s2n_connection *conn, s2n_blocked_status *blocked)
             }
         }
 
-        /* If the handshake has just ended, free up memory */
         if (ACTIVE_STATE(conn).writer == 'B') {
-            POSIX_GUARD(s2n_stuffer_resize(&conn->handshake.io, 0));
-
             /* Send any pending post-handshake messages */
             POSIX_GUARD(s2n_post_handshake_send(conn, blocked));
+
+            /* If the handshake has just ended, free up memory */
+            POSIX_GUARD(s2n_stuffer_resize(&conn->handshake.io, 0));
         }
     }
 
     *blocked = S2N_NOT_BLOCKED;
 
-    return 0;
+    return S2N_SUCCESS;
 }
