@@ -98,7 +98,7 @@ static int s2n_set_cert_chain_as_client(struct s2n_connection *conn)
 {
     if (s2n_config_get_num_default_certs(conn->config) > 0) {
         POSIX_GUARD(s2n_choose_sig_scheme_from_peer_preference_list(conn, &conn->handshake_params.server_sig_hash_algs,
-                                                               &conn->secure.client_cert_sig_scheme));
+                                                               &conn->handshake_params.client_cert_sig_scheme));
 
         struct s2n_cert_chain_and_key *cert = s2n_config_get_single_default_cert(conn->config);
         POSIX_ENSURE_REF(cert);
@@ -131,7 +131,7 @@ int s2n_cert_req_recv(struct s2n_connection *conn)
 
     s2n_cert_type cert_type = 0;
     POSIX_GUARD(s2n_recv_client_cert_preferences(in, &cert_type));
-    POSIX_GUARD(s2n_cert_type_to_pkey_type(cert_type, &conn->secure.client_cert_pkey_type));
+    POSIX_GUARD(s2n_cert_type_to_pkey_type(cert_type, &conn->handshake_params.client_cert_pkey_type));
 
     if (conn->actual_protocol_version == S2N_TLS12) {
         POSIX_GUARD(s2n_recv_supported_sig_scheme_list(in, &conn->handshake_params.server_sig_hash_algs));
