@@ -38,7 +38,7 @@ int s2n_tls13_mac_verify(struct s2n_tls13_keys *keys, struct s2n_blob *finished_
 
     S2N_ERROR_IF(!s2n_constant_time_equals(finished_verify->data, wire_verify->data, keys->size), S2N_ERR_BAD_MESSAGE);
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 /*
@@ -55,14 +55,14 @@ static int s2n_tls13_keys_init_with_ref(struct s2n_tls13_keys *handshake, s2n_hm
     POSIX_GUARD(s2n_blob_init(&handshake->derive_secret, derive, handshake->size));
     POSIX_GUARD(s2n_hmac_new(&handshake->hmac));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 int s2n_tls13_keys_from_conn(struct s2n_tls13_keys *keys, struct s2n_connection *conn)
 {
-    POSIX_GUARD(s2n_tls13_keys_init_with_ref(keys, conn->secure.cipher_suite->prf_alg, conn->secure.rsa_premaster_secret, conn->secure.master_secret));
+    POSIX_GUARD(s2n_tls13_keys_init_with_ref(keys, conn->secure.cipher_suite->prf_alg, conn->secrets.rsa_premaster_secret, conn->secrets.master_secret));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 int s2n_tls13_compute_ecc_shared_secret(struct s2n_connection *conn, struct s2n_blob *shared_secret) {
@@ -72,20 +72,15 @@ int s2n_tls13_compute_ecc_shared_secret(struct s2n_connection *conn, struct s2n_
     POSIX_GUARD(s2n_connection_get_ecc_preferences(conn, &ecc_preferences));
     POSIX_ENSURE_REF(ecc_preferences);
 
-    struct s2n_ecc_evp_params *server_key = &conn->secure.server_ecc_evp_params;
+    struct s2n_ecc_evp_params *server_key = &conn->kex_params.server_ecc_evp_params;
     POSIX_ENSURE_REF(server_key);
     POSIX_ENSURE_REF(server_key->negotiated_curve);
-    /* for now we do this tedious loop to find the matching client key selection.
-     * this can be simplified if we get an index or a pointer to a specific key */
-    struct s2n_ecc_evp_params *client_key = NULL;
-    for (size_t i = 0; i < ecc_preferences->count; i++) {
-        if (server_key->negotiated_curve->iana_id == ecc_preferences->ecc_curves[i]->iana_id) {
-            client_key = &conn->secure.client_ecc_evp_params[i];
-            break;
-        }
-    }
 
-    POSIX_ENSURE(client_key != NULL, S2N_ERR_BAD_KEY_SHARE);
+    struct s2n_ecc_evp_params *client_key  = &conn->kex_params.client_ecc_evp_params;
+    POSIX_ENSURE_REF(client_key);
+    POSIX_ENSURE_REF(client_key->negotiated_curve);
+
+    POSIX_ENSURE_EQ(server_key->negotiated_curve, client_key->negotiated_curve);
 
     if (conn->mode == S2N_CLIENT) {
         POSIX_GUARD(s2n_ecc_evp_compute_shared_secret_from_params(client_key, server_key, shared_secret));
@@ -93,7 +88,7 @@ int s2n_tls13_compute_ecc_shared_secret(struct s2n_connection *conn, struct s2n_
         POSIX_GUARD(s2n_ecc_evp_compute_shared_secret_from_params(server_key, client_key, shared_secret));
     }
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 /* Computes the ECDHE+PQKEM hybrid shared secret as defined in
@@ -102,16 +97,16 @@ int s2n_tls13_compute_pq_hybrid_shared_secret(struct s2n_connection *conn, struc
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(shared_secret);
 
-    /* conn->secure.server_ecc_evp_params should be set only during a classic/non-hybrid handshake */
-    POSIX_ENSURE_EQ(NULL, conn->secure.server_ecc_evp_params.negotiated_curve);
-    POSIX_ENSURE_EQ(NULL, conn->secure.server_ecc_evp_params.evp_pkey);
+    /* conn->kex_params.server_ecc_evp_params should be set only during a classic/non-hybrid handshake */
+    POSIX_ENSURE_EQ(NULL, conn->kex_params.server_ecc_evp_params.negotiated_curve);
+    POSIX_ENSURE_EQ(NULL, conn->kex_params.server_ecc_evp_params.evp_pkey);
 
-    struct s2n_kem_group_params *server_kem_group_params = &conn->secure.server_kem_group_params;
+    struct s2n_kem_group_params *server_kem_group_params = &conn->kex_params.server_kem_group_params;
     POSIX_ENSURE_REF(server_kem_group_params);
     struct s2n_ecc_evp_params *server_ecc_params = &server_kem_group_params->ecc_params;
     POSIX_ENSURE_REF(server_ecc_params);
 
-    struct s2n_kem_group_params *client_kem_group_params = conn->secure.chosen_client_kem_group_params;
+    struct s2n_kem_group_params *client_kem_group_params = &conn->kex_params.client_kem_group_params;
     POSIX_ENSURE_REF(client_kem_group_params);
     struct s2n_ecc_evp_params *client_ecc_params = &client_kem_group_params->ecc_params;
     POSIX_ENSURE_REF(client_ecc_params);
@@ -129,7 +124,7 @@ int s2n_tls13_compute_pq_hybrid_shared_secret(struct s2n_connection *conn, struc
     POSIX_ENSURE_REF(pq_shared_secret);
     POSIX_ENSURE_REF(pq_shared_secret->data);
 
-    const struct s2n_kem_group *negotiated_kem_group = conn->secure.server_kem_group_params.kem_group;
+    const struct s2n_kem_group *negotiated_kem_group = conn->kex_params.server_kem_group_params.kem_group;
     POSIX_ENSURE_REF(negotiated_kem_group);
     POSIX_ENSURE_REF(negotiated_kem_group->kem);
 
@@ -147,7 +142,7 @@ int s2n_tls13_compute_pq_hybrid_shared_secret(struct s2n_connection *conn, struc
 }
 
 static int s2n_tls13_pq_hybrid_supported(struct s2n_connection *conn) {
-    return conn->secure.server_kem_group_params.kem_group != NULL;
+    return conn->kex_params.server_kem_group_params.kem_group != NULL;
 }
 
 int s2n_tls13_compute_shared_secret(struct s2n_connection *conn, struct s2n_blob *shared_secret)
@@ -200,7 +195,6 @@ int s2n_tls13_handle_early_secret(struct s2n_connection *conn)
     return S2N_SUCCESS;
 }
 
-
 int s2n_tls13_handle_early_traffic_secret(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
@@ -263,7 +257,7 @@ int s2n_tls13_handle_handshake_master_secret(struct s2n_connection *conn)
 
     POSIX_GUARD(s2n_tls13_extract_handshake_secret(&secrets, &shared_secret));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 int s2n_tls13_handle_handshake_traffic_secret(struct s2n_connection *conn, s2n_mode mode)
@@ -331,7 +325,7 @@ int s2n_tls13_handle_handshake_traffic_secret(struct s2n_connection *conn, s2n_m
      */
     POSIX_GUARD(s2n_zero_sequence_number(conn, mode));
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 static int s2n_tls13_handle_application_secret(struct s2n_connection *conn, s2n_mode mode)
@@ -346,12 +340,12 @@ static int s2n_tls13_handle_application_secret(struct s2n_connection *conn, s2n_
     struct s2n_session_key *session_key;
     s2n_secret_type_t secret_type;
     if (mode == S2N_CLIENT) {
-        app_secret_data = conn->secure.client_app_secret;
+        app_secret_data = conn->secrets.client_app_secret;
         implicit_iv_data = conn->secure.client_implicit_iv;
         session_key = &conn->secure.client_key;
         secret_type = S2N_CLIENT_APPLICATION_TRAFFIC_SECRET;
     } else {
-        app_secret_data = conn->secure.server_app_secret;
+        app_secret_data = conn->secrets.server_app_secret;
         implicit_iv_data = conn->secure.server_implicit_iv;
         session_key = &conn->secure.server_key;
         secret_type = S2N_SERVER_APPLICATION_TRAFFIC_SECRET;
@@ -527,11 +521,11 @@ int s2n_update_application_traffic_keys(struct s2n_connection *conn, s2n_mode mo
 
     if (mode == S2N_CLIENT) {
         old_key = &conn->secure.client_key;
-        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secure.client_app_secret, keys.size));
+        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secrets.client_app_secret, keys.size));
         POSIX_GUARD(s2n_blob_init(&app_iv, conn->secure.client_implicit_iv, S2N_TLS13_FIXED_IV_LEN));
     } else {
         old_key = &conn->secure.server_key;
-        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secure.server_app_secret, keys.size));
+        POSIX_GUARD(s2n_blob_init(&old_app_secret, conn->secrets.server_app_secret, keys.size));
         POSIX_GUARD(s2n_blob_init(&app_iv, conn->secure.server_implicit_iv, S2N_TLS13_FIXED_IV_LEN));  
     }
 
